""" An empty module for hosting the element classes created on-the-fly.
"""
import sys
# from . import base

# Allow accessing all ontoweaver.Item classes defined in this module.
# all = base.All(sys.modules[__name__]) # FIXME circular import from base

